from ezcode.list.linked_list import DoublyLinkedList


class LRUCache:
    def __init__(self, capacity: int):
        self.capacity = capacity
        self.cache = dict()  # value = Node
        self.doubly_linked_list = DoublyLinkedList()
        
    def get(self, key):
        if key not in self.cache:
            return None
        node = self.cache[key]
        self.doubly_linked_list.detach_node(node)
        self.doubly_linked_list.add_node_to_head(node)
        return self.doubly_linked_list.algorithm.get_data(node)

    def put(self, key, value):
        if key not in self.cache:
            if self.doubly_linked_list.size == self.capacity:
                del self.cache[self.doubly_linked_list.tail.key]
                self.doubly_linked_list.detach_node(self.doubly_linked_list.tail)
            node = self.doubly_linked_list.algorithm.new_node(data=value)
            node.__dict__["key"] = key
            self.doubly_linked_list.add_node_to_head(node)
            self.cache[key] = node
        else:
            node = self.cache[key]
            self.doubly_linked_list.algorithm.set_data(node=node, data=value)
            self.doubly_linked_list.detach_node(node)
            self.doubly_linked_list.add_node_to_head(node)



import math


def _binary_search(array, begin: int, end: int, target, is_ascending=True, is_left_most=True):
    while begin < end:
        mid = begin + (end - begin) / 2
        mid = math.floor(mid) if is_left_most else math.ceil(mid)
        if (is_ascending and target < array[mid]) or (not is_ascending and target > array[mid]):
            end = mid - 1
        elif (is_ascending and target > array[mid]) or (not is_ascending and target < array[mid]):
            begin = mid + 1
        else:
            if is_left_most:
                end = mid
            else:
                begin = mid
    return begin if array[begin] == target else None


def binary_search(array, target, is_ascending=True, is_left_most=True):
    """ is_left_most is used when the array has duplicate entries """
    if not array:
        return None
    return _binary_search(array, 0, len(array) - 1, target, is_ascending, is_left_most)



from collections import deque
from typing import List


class DirectedGraph:
    class Node():
        def __init__(self, node_id=None, src_nodes: set = None, dst_nodes: set = None):
            self.node_id = node_id
            self.src_nodes = src_nodes  # set(Node)
            self.dst_nodes = dst_nodes  # set(Node)

    def __init__(self, edges: List[list] = None):
        self.nodes = None  # dict(node_id, Node)
        self.sorted_node_ids = None     # for print
        self.node_id_index_map = None   # for print
        if edges:
            self.build(edges=edges)

    def __len__(self):
        return len(self.nodes)

    def __str__(self, column_size=2, mark="*"):
        def get_cell(column_size, item=""):
            return str(item) + " " * (column_size - len(str(item)))
        matrix = self.generate_matrix()
        for node_id in self.sorted_node_ids:
            if len(node_id) + 1 > column_size:
                column_size = len(node_id) + 1
        output = get_cell(column_size)
        for node_id in self.sorted_node_ids:
            output += get_cell(column_size, node_id)
        output += "\n"
        for row in range(len(self)):
            node_id = self.sorted_node_ids[row]
            output += get_cell(column_size, node_id)
            for col in range(len(self)):
                output += get_cell(column_size, mark) if matrix[row][col] else get_cell(column_size)
            output += "\n"
        return output

    def build(self, edges: List[list]):
        if not self.nodes:
            self.nodes = dict()
        for (src_id, dst_id) in edges:
            if src_id:
                if src_id not in self.nodes:
                    self.nodes[src_id] = self.Node(node_id=src_id)
                src_node = self.nodes[src_id]
            else:
                src_node = None
            if dst_id:
                if dst_id not in self.nodes:
                    self.nodes[dst_id] = self.Node(node_id=dst_id)
                dst_node = self.nodes[dst_id]
            else:
                dst_node = None
            if src_node and dst_node:
                if not src_node.dst_nodes:
                    src_node.dst_nodes = set()
                src_node.dst_nodes.add(dst_node)
                if not dst_node.src_nodes:
                    dst_node.src_nodes = set()
                dst_node.src_nodes.add(src_node)
        # For print
        self.sorted_node_ids = sorted(self.nodes.keys())
        self.node_id_index_map = dict()
        for index, node_id in enumerate(self.sorted_node_ids):
            self.node_id_index_map[node_id] = index

    def print(self):
        print(self, end="")

    def generate_matrix(self):
        matrix = [[False] * len(self) for _ in range(len(self))]
        for node_id in self.sorted_node_ids:
            node = self.nodes[node_id]
            if node.dst_nodes:
                for dst_node in node.dst_nodes:
                    src_index = self.node_id_index_map[node.node_id]
                    dst_index = self.node_id_index_map[dst_node.node_id]
                    matrix[src_index][dst_index] = True
        return matrix

    def copy_nodes(self):
        new_nodes = dict()
        for node_id, node in self.nodes.items():
            if node.src_nodes:
                new_src_nodes = set()
                for src_node in node.src_nodes:
                    if src_node.node_id not in new_nodes:
                        new_nodes[src_node.node_id] = self.Node(node_id=src_node.node_id)
                    new_src_nodes.add(new_nodes[src_node.node_id])
            else:
                new_src_nodes = None
            if node.dst_nodes:
                new_dst_nodes = set()
                for dst_node in node.dst_nodes:
                    if dst_node.node_id not in new_nodes:
                        new_nodes[dst_node.node_id] = self.Node(node_id=dst_node.node_id)
                    new_dst_nodes.add(new_nodes[dst_node.node_id])
            else:
                new_dst_nodes = None
            if node_id not in new_nodes:
                new_nodes[node_id] = self.Node(node_id, new_src_nodes, new_dst_nodes)
            else:
                new_nodes[node_id].src_nodes = new_src_nodes
                new_nodes[node_id].dst_nodes = new_dst_nodes
        return new_nodes

    def topological_order(self):
        topological_order = list()
        nodes_without_dst = deque()
        for _, node in self.copy_nodes().items():
            if not node.dst_nodes or len(node.dst_nodes) == 0:
                nodes_without_dst.append(node)
        while len(nodes_without_dst) > 0:
            node = nodes_without_dst.pop()
            topological_order.append(node.node_id)
            if node.src_nodes:
                for src_node in node.src_nodes:
                    src_node.dst_nodes.remove(node)
                    if not src_node.dst_nodes:
                        nodes_without_dst.append(src_node)
        return topological_order

    def is_directed_acyclic_graph(self):
        return len(self.topological_order()) == len(self)


            





from ezcode.array.matrix import init_matrix


def longest_common_subsequence(array_1: list, array_2: list) -> list:
    row_len, col_len = len(array_1) + 1, len(array_2) + 1  # Add one dummy row and one dummy column
    dp = init_matrix(row=row_len, col=col_len, init=0)
    for row in range(1, row_len):
        for col in range(1, col_len):
            dp[row][col] = dp[row - 1][col - 1] + 1 if array_1[row - 1] == array_2[col - 1] else max(dp[row - 1][col], dp[row][col - 1])
    row, col, lcs = row_len - 1, col_len - 1, list()
    while row > 0 and col > 0:
        if array_1[row - 1] == array_2[col - 1]:
            lcs.append(array_1[row - 1])
            row, col = row - 1, col - 1
        else:
            row, col = (row - 1, col) if dp[row - 1][col] >= dp[row][col - 1] else (row, col - 1)
    return lcs[::-1]


def longest_common_subarray(array_1: list, array_2: list) -> list:
    row_len, col_len = len(array_1) + 1, len(array_2) + 1  # Add one dummy row and one dummy column
    dp = init_matrix(row=row_len, col=col_len, init=0)
    lcs_size, row_end = 0, 0
    for row in range(1, row_len):
        for col in range(1, col_len):
            if array_1[row - 1] == array_2[col - 1]:
                dp[row][col] = dp[row - 1][col - 1] + 1
                if dp[row][col] > lcs_size:
                    lcs_size, row_end = dp[row][col], row
    lcs = list()
    while lcs_size > 0:
        lcs.append(array_1[row_end - 1])
        row_end -= 1
        lcs_size -= 1
    return lcs[::-1]



import sys

from ezcode.tree.const import DATA_NAME, LEFT_NAME, RIGHT_NAME


class BinaryTreeAlgorithm:
    """ Recursion Helpers """

    def __init__(self, data_name: str = DATA_NAME, left_name: str = LEFT_NAME, right_name: str = RIGHT_NAME):
        self.data_name = data_name
        self.left_name = left_name
        self.right_name = right_name

    def pre_order(self, root, result: list):
        if root is not None:
            result.append(root.__dict__[self.data_name])
            self.pre_order(root.__dict__[self.left_name], result)
            self.pre_order(root.__dict__[self.right_name], result)

    def in_order(self, root, result: list):
        if root is not None:
            self.pre_order(root.__dict__[self.left_name], result)
            result.append(root.__dict__[self.data_name])
            self.pre_order(root.__dict__[self.right_name], result)
    
    def post_order(self, root, result: list):
        if root is not None:
            self.pre_order(root.__dict__[self.left_name], result)
            self.pre_order(root.__dict__[self.right_name], result)
            result.append(root.__dict__[self.data_name])

    def depth(self, root):
        if root is None:
            return 0
        return max(self.depth(root.__dict__[self.left_name]), self.depth(root.__dict__[self.right_name])) + 1

    def subtree_sum_extremum(self, root, extremum_func):
        if root is None:
            return 0, 0
        left_sum_extremum, left_sum = self.subtree_sum_extremum(root.__dict__[self.left_name], extremum_func)
        right_sum_extremum, right_sum = self.subtree_sum_extremum(root.__dict__[self.right_name], extremum_func)
        my_sum = left_sum + right_sum + root.__dict__[self.data_name]
        return extremum_func(my_sum, left_sum_extremum, right_sum_extremum), my_sum
    
    def subtree_avg_extremum(self, root, extremum_func):
        if root is None:
            return 0, 0, 0
        left_avg_extremum, left_sum, left_size = self.subtree_avg_extremum(root.__dict__[self.left_name], extremum_func)
        right_avg_extremum, right_sum, right_size = self.subtree_avg_extremum(root.__dict__[self.right_name], extremum_func)
        my_sum = left_sum + right_sum + root.__dict__[self.data_name]
        my_size = left_size + right_size + 1
        my_average = my_sum / my_size
        return extremum_func(my_average, left_avg_extremum, right_avg_extremum), my_sum, my_size
    
    def lowest_common_ancestor(self, root, node_1, node_2):
        if root is None or root == node_1 or root == node_2:
            return root
        left_ancestor = self.lowest_common_ancestor(root.__dict__[self.left_name], node_1, node_2)
        right_ancestor = self.lowest_common_ancestor(root.__dict__[self.right_name], node_1, node_2)
        if left_ancestor is None and right_ancestor is None:
            return None
        if left_ancestor is not None and right_ancestor is not None:
            return root
        return left_ancestor if left_ancestor is not None else right_ancestor

    def is_balanced(self, root) -> (bool, int):
        if root is None:
            return True, 0
        left_balanced, left_depth = self.is_balanced(root.__dict__[self.left_name])
        right_balanced, right_depth = self.is_balanced(root.__dict__[self.right_name])
        return left_balanced and right_balanced and abs(left_depth - right_depth) <= 1, max(left_depth, right_depth) + 1
    
    def is_copied(self, root_1, root_2):
        if not root_1 and not root_2:
            return True
        if not root_1 or not root_2:
            return False
        if root_1.__dict__[self.data_name] != root_2.__dict__[self.data_name]:
            return False
        return self.is_copied(root_1.__dict__[self.left_name], root_2.__dict__[self.left_name]) and \
               self.is_copied(root_1.__dict__[self.right_name], root_2.__dict__[self.right_name])
    
    def max_path_sum(self, root):
        if root is None:
            return -sys.maxsize, 0  # path sum max，max half + node value
        left_max, l_half = self.max_path_sum(root.__dict__[self.left_name])  # left path max, left non-negative max half
        right_max, r_half = self.max_path_sum(root.__dict__[self.right_name])  # right path max, right non-negative max half
        return max(left_max, right_max, root.__dict__[self.data_name] + l_half + r_half), \
               max(root.__dict__[self.data_name] + max(l_half, r_half), 0)





    


import math
import random
import sys

from collections import deque
from ezcode.tree.const import DATA_NAME, LEFT_NAME, RIGHT_NAME
from ezcode.tree.const import LEFT_WING, RIGHT_WING, LEFT_WING_HEAD, RIGHT_WING_HEAD, LEFT_WING_TAIL, RIGHT_WING_TAIL


class BinaryTreePrinter:
    class IndexedNode:
        def __init__(self, index, node):
            self.index = index
            self.node = node

    def __init__(self,
        data_name: str = DATA_NAME, left_name: str = LEFT_NAME, right_name: str = RIGHT_NAME,
        left_wing: str = LEFT_WING, right_wing: str = RIGHT_WING,
        left_wing_head: str = LEFT_WING_HEAD, right_wing_head: str = RIGHT_WING_HEAD,
        left_wing_tail: str = LEFT_WING_TAIL, right_wing_tail: str = RIGHT_WING_TAIL
    ):
        self.data_name = data_name
        self.left_name = left_name
        self.right_name = right_name
        self.left_wing = left_wing
        self.right_wing = right_wing
        self.left_wing_head = left_wing_head 
        self.right_wing_head = right_wing_head
        self.left_wing_tail = left_wing_tail
        self.right_wing_tail = right_wing_tail
        self.tree_depth = 0
        self.max_data_string_length = 0
        self.char_map = None

    def _align_to_center(self, node, is_last_line):
        """ e.g.
        max data string length: 10
        data length: 3
        pre padding length: 4
        post padding length: 3
        pre padding char: -
        post padding char: -
        left wing tail: <
        right wing tail: >
        ---<ABC>--
        """
        pre_padding_length = math.ceil((self.max_data_string_length - len(str(node.__dict__[self.data_name]))) / 2) # even length padding more, use larger mid index
        post_padding_length = self.max_data_string_length - len(str(node.__dict__[self.data_name])) - pre_padding_length
        pre_padding_char = " " * len(self.left_wing) if is_last_line or node.__dict__[self.left_name] is None else self.left_wing
        post_padding_char = " " * len(self.right_wing) if is_last_line or node.__dict__[self.right_name] is None else self.right_wing
        pre_padding_string = f"{pre_padding_char * (pre_padding_length - 1)}{self.left_wing_tail}"
        post_padding_string = f"{self.right_wing_tail}{post_padding_char * (post_padding_length - 1)}"
        
        return f"{pre_padding_string}{str(node.__dict__[self.data_name])}{post_padding_string}"

    def _set_data_string(self, line_to_print, indexed_node, depth):

        def _get_mid_index(depth, index, line_length):
            slot_count = pow(2, depth)
            slot_id = index * 2 - slot_count + 1
            return int(line_length * slot_id / slot_count)

        data_string = self._align_to_center(indexed_node.node, depth == self.tree_depth)
        start_index = _get_mid_index(depth, indexed_node.index, len(line_to_print)) - int(self.max_data_string_length / 2)
        line_to_print[start_index:(start_index + self.max_data_string_length)] = list(data_string)
        if depth != self.tree_depth:
            if indexed_node.node.__dict__[self.left_name] is not None:
                left_child_mid_index = _get_mid_index(depth + 1, indexed_node.index * 2, len(line_to_print))
                line_to_print[(left_child_mid_index + 1):start_index] = list(self.left_wing * (start_index - left_child_mid_index - 1))
                line_to_print[left_child_mid_index] = self.left_wing_head
            if indexed_node.node.__dict__[self.right_name] is not None:
                right_child_mid_index = _get_mid_index(depth + 1, indexed_node.index * 2 + 1, len(line_to_print))
                line_to_print[(start_index + self.max_data_string_length):right_child_mid_index] = list(self.right_wing * (right_child_mid_index - start_index - self.max_data_string_length))
                line_to_print[right_child_mid_index] = self.right_wing_head

    def _collect_tree_info(self, node):
        """ return depth and update max_data_string_length """
        if node is None:
            return 0
        data_str_len = len(str(node.__dict__[self.data_name])) + len(self.left_wing_head) + len(self.right_wing_head) + len(self.left_wing_tail) + len(self.right_wing_tail)
        self.max_data_string_length = max(data_str_len, self.max_data_string_length)
        left_depth = self._collect_tree_info(node.__dict__[self.left_name]) if node.__dict__[self.left_name] is not None else 0
        right_depth = self._collect_tree_info(node.__dict__[self.right_name]) if node.__dict__[self.right_name] is not None else 0
        return max(left_depth, right_depth) + 1

    def _make_char_map(self, node):
        self.char_map = list()
        if node is None:
            return
        self.tree_depth = self._collect_tree_info(node)
        last_line_data_count = pow(2, self.tree_depth - 1)
        last_line_string_length = last_line_data_count * (self.max_data_string_length + 1) - 1 # +one space in between
        """
        [PrePadding][LeftWingTail][Data][RightWingTail][PostPadding][SPACE][PrePadding][LeftWingTail][Data][RightWingTail][PostPadding][SPACE]...
        """
        queue = deque()
        queue.append(BinaryTreePrinter.IndexedNode(1, node))
        children_found, children_left, depth = 0, 1, 1
        line_to_print = list(" " * last_line_string_length)  # we have to make it a list because string is immutable
        while len(queue) > 0:
            indexed_node = queue.popleft()
            children_left -= 1
            self._set_data_string(line_to_print, indexed_node, depth)
            if indexed_node.node.__dict__[self.left_name] is not None:
                queue.append(BinaryTreePrinter.IndexedNode(int(indexed_node.index * 2), indexed_node.node.__dict__[self.left_name]))
                children_found += 1
            if indexed_node.node.__dict__[self.right_name] is not None:
                queue.append(BinaryTreePrinter.IndexedNode(int(indexed_node.index * 2) + 1, indexed_node.node.__dict__[self.right_name]))
                children_found += 1
            if children_left == 0:
                self.char_map.append(line_to_print)
                children_left = children_found
                children_found = 0
                line_to_print = list(" " * last_line_string_length)
                depth += 1

    def to_string(self, node, trim_left=True, trim_right=True):
        self._make_char_map(node)
        start_print_index, end_print_index = 0, len(self.char_map[0])
        if trim_left:  # trim left blank
            for line in self.char_map:
                last_non_blank_index_from_begin = 0
                for char in line:
                    if char == " ":
                        last_non_blank_index_from_begin += 1
                    else:
                        break
                if start_print_index == 0 or last_non_blank_index_from_begin < start_print_index:
                    start_print_index = last_non_blank_index_from_begin
        if trim_right:  # trim right blank
            for line in self.char_map:
                last_non_blank_index_from_end = len(self.char_map[0])
                for char in line[::-1]:
                    if char == " ":
                        last_non_blank_index_from_end -= 1
                    else:
                        break
                if end_print_index == len(self.char_map[0]) or last_non_blank_index_from_end > end_print_index:
                    end_print_index = last_non_blank_index_from_end
        output_string = ""
        for line in self.char_map:
            output_string += "".join(line[start_print_index:end_print_index]) + "\n"
        return output_string

    def print(self, node, trim_left=True, trim_right=True):
        print("\n" + self.to_string(node, trim_left, trim_right))






def swap(array: list, i: int, j: int):
    if i != j:
        tmp = array[i]
        array[i] = array[j]
        array[j] = tmp


def is_copied(array_1, array_2):
    if type(array_1) is not list and type(array_2) is not list:
        return array_1 == array_2
    elif type(array_1) is list and type(array_2) is list:
        if len(array_1) != len(array_2):
            return False
        result = True
        for subarray_1, subarray_2 in zip(array_1, array_2):
            result &= is_copied(subarray_1, subarray_2)
        return result
    else:
        return False


def copy(array: list) -> list:
    return [copy(subarray) for subarray in array] if type(array) is list else array


def delete(array: list, items_to_delete: set):
    count = 0
    for index, data in enumerate(array):
        if data in items_to_delete:
            count += 1
        elif count > 0:
            array[index - count] = data
    for _ in range(count):
        array.pop()
        

def array_to_string(array, indent: str = "    "):
    def _array_to_string(array: list, depth: int, result: list):
        if type(array) is list:
            no_list_found = True
            for subarray in array:
                if type(subarray) is list:
                    no_list_found = False
            if no_list_found:
                result.append(f"{indent * depth}{array},\n")
            else:
                result.append(f"{indent * depth}[\n")
                for subarray in array:
                    _array_to_string(subarray, depth + 1, result)
                result.append(f"{indent * depth}]")
                result.append(",\n" if depth > 0 else "\n")
        else:
            result.append(f"{indent * depth}{array},\n")
    result = list()
    _array_to_string(array, 0, result)
    return "".join(result)


def print_array(array, indent: str = "    "):
    print(array_to_string(array, indent), end="")


from typing import List
from collections import deque
from ezcode.array.heap import PriorityMap
from ezcode.graph import NegativeCycleExist


class UndirectedGraph:
    def __init__(self, edges: List[list] = None, weights: list = None):
        self.nodes = dict()  # <node_id, <node_id, weight>>
        self.is_weighted: bool = weights is not None
        self.sorted_node_ids: list = None     # for print
        self.node_id_index_map: dict = None   # for print
        self.mark = "*"                       # for print
        if edges:
            self.build(edges=edges, weights=weights)

    def __len__(self):
        return len(self.nodes)

    def build(self, edges: List[list], weights: list = None):
        if weights is None:
            weights = [1] * len(edges)
        for (n1, n2), weight in zip(edges, weights):
            if n1 not in self.nodes:
                self.nodes[n1] = dict()
            if n2 not in self.nodes:
                self.nodes[n2] = dict()
            self.nodes[n1][n2] = self.nodes[n2][n1] = weight
        # For print
        self.sorted_node_ids = sorted(self.nodes.keys())
        self.node_id_index_map = dict()
        for index, node_id in enumerate(self.sorted_node_ids):
            self.node_id_index_map[node_id] = index

    def __str__(self):
        def get_cell(cell_size, item=""):
            if item == 1 and not self.is_weighted:
                item = self.mark
            return str(item) + (" " * (cell_size - len(str(item))))

        max_cell_size = 0
        for node_id in self.sorted_node_ids:
            max_cell_size = max(max_cell_size, len(str(node_id)))
        if self.is_weighted:
            for edges in self.nodes.values():
                for weight in edges.values():
                     max_cell_size = max(max_cell_size, len(str(weight)))
        max_cell_size += 2  # Add two spaces in between
        output = get_cell(max_cell_size)
        for node_id in self.sorted_node_ids:
            output += get_cell(max_cell_size, node_id)
        output += "\n"
        for row in range(len(self)):
            n1 = self.sorted_node_ids[row]
            output += get_cell(max_cell_size, n1)
            for col in range(len(self)):
                n2 = self.sorted_node_ids[col]
                output += get_cell(max_cell_size, self.nodes[n1][n2]) if n2 in self.nodes[n1] else get_cell(max_cell_size)
            output += "\n"
        return output

    def print(self):
        print(self, end="")

    def dfs_path_value(self, src_node_id, dst_node_id, visited = set(), self_loop_value=0, path_value_init=float("inf"), path_value_func=lambda a, b: a + b, min_max_func=min):
        if src_node_id == dst_node_id:
            return self_loop_value
        top_path_value = path_value_init
        for node_id, weight in self.nodes[src_node_id].items():
            if node_id not in visited:
                visited.add(node_id)
                path_value = self.dfs_path_value(node_id, dst_node_id, visited, self_loop_value, path_value_init, path_value_func, min_max_func)
                visited.remove(node_id)
                top_path_value = min_max_func(top_path_value, path_value_func(weight, path_value))
        return top_path_value

    def dijkstra(self, src_node_id, dst_node_id, self_loop_value=0, path_value_init=float("inf"), path_value_func=lambda a, b: a + b, min_max_func=min):
        """ Positive Weight Only: O(V + E*logV). On dense graphs, dijkstra is faster than spfa """
        path_values, visited = dict(), set()
        min_heap = True if min_max_func == min else False
        candidates = PriorityMap({src_node_id:self_loop_value}, min_heap=min_heap)
        for node_id in self.nodes.keys():
            path_values[node_id] = self_loop_value if node_id == src_node_id else path_value_init
        while len(candidates) > 0:
            top_path_value, top_node_id = candidates.pop()
            visited.add(top_node_id)
            for relax_node_id, weight in self.nodes[top_node_id].items():
                if relax_node_id not in visited:
                    path_values[relax_node_id] = min_max_func(path_values[relax_node_id], path_value_func(top_path_value, weight))
                    candidates.push(path_values[relax_node_id], relax_node_id)
        return path_values[dst_node_id]

    def spfa(self, src_node_id, dst_node_id, self_loop_value=0, path_value_init=float("inf"), path_value_func=lambda a, b: a + b, min_max_func=min, check_negative_weight=False):
        """ Improved Bellman Ford Algorithm: can handle Negative Weight and detect Negative Cycle: worst case O(V*E), average O(E), sparse graphs O(V^2) """
        path_values, queue, queue_set = dict(), deque([src_node_id]), set([src_node_id])
        enqueue_counters = dict() if check_negative_weight else None
        for node_id in self.nodes.keys():
            path_values[node_id] = self_loop_value if node_id == src_node_id else path_value_init
            if check_negative_weight:
                enqueue_counters[node_id] = 1 if node_id == src_node_id else 0
        while len(queue) > 0:
            node_id = queue.popleft()
            queue_set.remove(node_id)
            for relax_node_id, weight in self.nodes[node_id].items():
                new_path_value = path_value_func(path_values[node_id], weight)
                if (min_max_func == min and new_path_value < path_values[relax_node_id]) or (min_max_func == max and new_path_value > path_values[relax_node_id]):
                    path_values[relax_node_id] = new_path_value
                    if relax_node_id not in queue_set:
                        queue.append(relax_node_id)
                        queue_set.add(relax_node_id)
                        if check_negative_weight:
                            enqueue_counters[relax_node_id] += 1
                            if enqueue_counters[relax_node_id] > len(self.nodes):
                                raise NegativeCycleExist()
        return path_values[dst_node_id]




        



















from ezcode.list.const import DATA_NAME, NEXT_NAME, PREV_NAME


class SinglyLinkedListAlgorithm:
    class FakeNode(object):
        def __init__(self):
           pass

    def __init__(self, data_name: str = DATA_NAME, next_name: str = NEXT_NAME):
        self.data_name = data_name
        self.next_name = next_name

    def new_node(self, data=None, next_node=None):
        node = self.FakeNode()
        node.__dict__ = {self.data_name: data, self.next_name: next_node}
        return node

    def has_next(self, node, steps: int = 1) -> bool:
        if not node:
            return False
        next_node = node
        for _ in range(steps):
            next_node = next_node.__dict__[self.next_name]
            if not next_node:
                return False
        return True

    def get_next(self, node, steps: int = 1):
        next_node = node
        for _ in range(steps):
            next_node = next_node.__dict__[self.next_name]
        return next_node

    def set_next(self, node, next_node=None):
        node.__dict__[self.next_name] = next_node

    def get_data(self, node):
        return node.__dict__[self.data_name]

    def set_data(self, node, data):
        node.__dict__[self.data_name] = data

    def reverse(self, previous_node, current_node):
        """ head = reverse(head, head.next) """
        if current_node:
            head = self.reverse(current, self.get_next(current))
            self.set_next(node=current, next_node=previous)
            self.set_next(node=previous, next_node=None)
            return head
        else:
            return previous


class DoublyLinkedListAlgorithm:
    class FakeNode(object):
        def __init__(self):
           pass

    def __init__(self, data_name: str = DATA_NAME, next_name: str = NEXT_NAME, prev_name: str = PREV_NAME):
        self.data_name = data_name
        self.next_name = next_name
        self.prev_name = prev_name

    def new_node(self, data=None, next_node=None, prev_node=None):
        node = self.FakeNode()
        node.__dict__ = {self.data_name: data, self.next_name: next_node, self.prev_name: prev_node}
        return node

    def has_next(self, node, steps: int = 1) -> bool:
        if not node:
            return False
        next_node = node
        for _ in range(steps):
            next_node = next_node.__dict__[self.next_name]
            if not next_node:
                return False
        return True

    def has_prev(self, node, steps: int = 1) -> bool:
        if not node:
            return False
        prev_node = node
        for _ in range(steps):
            prev_node = prev_node.__dict__[self.prev_name]
            if not prev_node:
                return False
        return True

    def get_next(self, node, steps: int = 1):
        next_node = node
        for _ in range(steps):
            next_node = next_node.__dict__[self.next_name]
        return next_node

    def get_prev(self, node, steps: int = 1):
        prev_node = node
        for _ in range(steps):
            prev_node = prev_node.__dict__[self.prev_name]
        return prev_node

    def set_next(self, node, next_node=None):
        node.__dict__[self.next_name] = next_node

    def set_prev(self, node, prev_node=None):
        node.__dict__[self.prev_name] = prev_node

    def get_data(self, node):
        return node.__dict__[self.data_name]

    def set_data(self, node, data):
        node.__dict__[self.data_name] = data




# Tree Node
DATA_NAME = "data"
LEFT_NAME = "left"
RIGHT_NAME = "right"

# Printer
LEFT_WING = "─"
RIGHT_WING = "─"
LEFT_WING_HEAD = "┌"
RIGHT_WING_HEAD = "┐"
LEFT_WING_TAIL = "("
RIGHT_WING_TAIL = ")"



from __future__ import annotations

import random

from collections import deque

from ezcode.tree.const import DATA_NAME, LEFT_NAME, RIGHT_NAME
from ezcode.tree.const import LEFT_WING, RIGHT_WING, LEFT_WING_HEAD, RIGHT_WING_HEAD, LEFT_WING_TAIL, RIGHT_WING_TAIL
from ezcode.tree.algorithm import BinaryTreeAlgorithm
from ezcode.tree.printer import BinaryTreePrinter


class BinaryTree(object):

    class FakeNode(object):
        def __init__(self):
           pass

    def __init__(self, root=None, data_name: str = DATA_NAME, left_name: str = LEFT_NAME, right_name: str = RIGHT_NAME):
        self.root = root
        self.data_name = data_name
        self.left_name = left_name
        self.right_name = right_name
        self.algorithm = BinaryTreeAlgorithm(data_name, left_name, right_name)

    def new_node(self, data):
        node = self.FakeNode()
        node.__dict__ = {self.data_name: data, self.left_name: None, self.right_name: None}
        return node
    
    def node_data(self, node):
        return node.__dict__[self.data_name]

    def print(self,
        left_wing: str = LEFT_WING, right_wing: str = RIGHT_WING,
        left_wing_head: str = LEFT_WING_HEAD, right_wing_head: str = RIGHT_WING_HEAD,
        left_wing_tail: str = LEFT_WING_TAIL, right_wing_tail: str = RIGHT_WING_TAIL
    ):
        BinaryTreePrinter(
            data_name=self.data_name, left_name=self.left_name, right_name=self.right_name,
            left_wing=left_wing, right_wing=right_wing,
            left_wing_head=left_wing_head, right_wing_head=right_wing_head,
            left_wing_tail=left_wing_tail, right_wing_tail=right_wing_tail
        ).print(self.root)

    def depth(self):
        return self.algorithm.depth(self.root)

    def is_balanced(self) -> bool:
        return self.algorithm.is_balanced(self.root)[0]

    def traversal(self, mode="pre-order"):
        result = list()
        if mode not in ["pre-order", "in-order", "post-order"]:
            raise ValueError(f"mode = \"{mode}\" is not supported, please choose from [\"pre-order\", \"in-order\", \"post-order\"]")
        elif mode == "pre-order":
            self.algorithm.pre_order(self.root, result)
        elif mode == "in-order":
            self.algorithm.in_order(self.root, result)
        elif mode == "post-order":
            self.algorithm.post_order(self.root, result)
        return result

    def subtree(self, mode="sum-min"):
        if mode not in ["sum-min", "sum-max", "avg-min", "avg-max"]:
            raise ValueError(f"mode = \"{mode}\" is not supported, please choose from [\"sum-min\", \"sum-max\", \"avg-min\", \"avg-max\"]")
        elif mode == "sum-min":
            return self.algorithm.subtree_sum_extremum(self.root, min)[0]
        elif mode == "sum-max":
            return self.algorithm.subtree_sum_extremum(self.root, max)[0]
        elif mode == "avg-min":
            return self.algorithm.subtree_avg_extremum(self.root, min)[0]
        else:
            return self.algorithm.subtree_avg_extremum(self.root, max)[0]

    def lowest_common_ancestor(self, nodes):
        if not nodes:
            return None
        if len(nodes) == 1:
            return nodes[0]
        ancestor = nodes[0]
        for node in nodes[1:]:
            ancestor = self.algorithm.lowest_common_ancestor(self.root, ancestor, node)
        return ancestor

    def max_path_sum(self):
        return self.algorithm.max_path_sum(self.root)[0]

    def serialize(self, delimiter: str = ",") -> str:
        if not self.root:
            return ""
        sequence = [self.root.__dict__[self.data_name]]
        queue = deque([self.root])
        while len(queue) > 0:
            node = queue.popleft()
            if node:
                queue.append(node.__dict__[self.left_name])
                queue.append(node.__dict__[self.right_name])
                if not node.__dict__[self.left_name]:
                    sequence.append(None)
                else:
                    sequence.append(node.__dict__[self.left_name].__dict__[self.data_name])
                if not node.__dict__[self.right_name]:
                    sequence.append(None)
                else:
                    sequence.append(node.__dict__[self.right_name].__dict__[self.data_name])
        return delimiter.join([str(x) for x in sequence])

    def deserialize(self, formatter, string: str, delimiter: str = ","):
        sequence = string.split(delimiter)
        if not sequence or not sequence[0]:
            return None
        root = self.new_node(formatter(sequence[0]))
        index, queue = 1, deque([root])
        while len(queue) > 0:
            node = queue.popleft()
            if node:
                left = self.new_node(formatter(sequence[index])) if sequence[index] != "None" else None
                right = self.new_node(formatter(sequence[index + 1])) if sequence[index + 1] != "None" else None
                queue.append(left)
                queue.append(right)
                node.__dict__[self.left_name], node.__dict__[self.right_name] = left, right
                index += 2
        return BinaryTree(root, self.data_name, self.left_name, self.right_name)

    def is_copied(self, tree: BinaryTree) -> bool:
        return self.algorithm.is_copied(self.root, tree.root)

    def copy(self) -> BinaryTree:
        if self.root is None:
            return BinaryTree(None, self.data_name, self.left_name, self.right_name)
        self_queue = deque([self.root])
        other_root = self.new_node(data=self.root.__dict__[self.data_name])
        other_queue = deque([other_root])
        while len(self_queue) > 0:
            self_node = self_queue.popleft()
            other_node = other_queue.popleft()
            if self_node:
                self_queue.append(self_node.__dict__[self.left_name])
                self_queue.append(self_node.__dict__[self.right_name])
                if self_node.__dict__[self.left_name]:
                    other_node.__dict__[self.left_name] = self.new_node(data=self_node.__dict__[self.left_name].__dict__[self.data_name])
                if self_node.__dict__[self.right_name]:
                    other_node.__dict__[self.right_name] = self.new_node(data=self_node.__dict__[self.right_name].__dict__[self.data_name])
                other_queue.append(other_node.__dict__[self.left_name])
                other_queue.append(other_node.__dict__[self.right_name])
        return BinaryTree(other_root, self.data_name, self.left_name, self.right_name)
        

class RandomBinaryTree(BinaryTree):
    def __init__(self,
        data_name: str = DATA_NAME, left_name: str = LEFT_NAME, right_name: str = RIGHT_NAME,
        size: int = 0, lower_bound: int = 0, upper_bound: int = 0
    ):
        super().__init__(None, data_name, left_name, right_name)
        self.size = size
        self.lower_bound = lower_bound
        self.upper_bound = upper_bound
        self.make_tree()

    def add_node(self, data):
        def _add_node(node, data):
            if random.randint(0, 1) == 0:
                if node.__dict__[self.left_name] is None:
                    node.__dict__[self.left_name] = self.new_node(data)
                else:
                    _add_node(node.__dict__[self.left_name], data)
            else:
                if node.__dict__[self.right_name] is None:
                    node.__dict__[self.right_name] = self.new_node(data)
                else:
                    _add_node(node.__dict__[self.right_name], data)

        if self.root is None:
            self.root = self.new_node(data)
        else:
            _add_node(self.root, data)

    def make_tree(self):
        self.root = None
        for _ in range(self.size):
            self.add_node(random.randint(self.lower_bound, self.upper_bound))



from __future__ import annotations

from ezcode.list.const import DATA_NAME, NEXT_NAME, PREV_NAME, FORWARD_LINK, BACKWARD_LINK, BIDIRECTION_LINK
from ezcode.list.algorithm import SinglyLinkedListAlgorithm, DoublyLinkedListAlgorithm
from ezcode.list.printer import SinglyLinkedListPrinter, DoublyLinkedListPrinter


class SinglyLinkedList(object):
    def __init__(self, head=None, data_name: str = DATA_NAME, next_name: str = NEXT_NAME):
        self.head = head
        self.algorithm = SinglyLinkedListAlgorithm(data_name, next_name)
        self.size = self.calculate_size()

    def __len__(self):
        return self.size

    def __str__(self):
        return self.to_string()

    def calculate_size(self) -> int:
        """ O(N) """
        size, node = 0, self.head
        while node:
            size, node = size + 1, self.algorithm.get_next(node)
        return size

    def to_string(self, reverse=False, include_end=True,
        forward_link: str = FORWARD_LINK, backward_link: str = BACKWARD_LINK
    ):
        printer = SinglyLinkedListPrinter(self.algorithm, forward_link, backward_link)
        return printer.to_string(self.head, reverse, include_end)

    def print(self, reverse=False, include_end=True,
        forward_link: str = FORWARD_LINK, backward_link: str = BACKWARD_LINK
    ):
        print(self.to_string(reverse, include_end, forward_link, backward_link))

    def add_to_head(self, data):
        """ O(1) """
        new_node = self.algorithm.new_node(data=data, next_node=self.head)
        self.head = new_node
        self.size += 1

    def peek_head(self):
        """ O(1) """
        if not self.head:
            raise IndexError("Peek head at an empty list")
        return self.algorithm.get_data(self.head)

    def pop_head(self):
        """ O(1) """
        if not self.head:
            raise IndexError("Pop head from an empty list")
        data = self.algorithm.get_data(self.head)
        self.head = self.algorithm.get_next(self.head)
        self.size -= 1
        return data

    def to_array(self):
        array, node = list(), self.head
        while node:
            array.append(self.algorithm.get_data(node))
            node = self.algorithm.get_next(node)
        return array

    def copy(self):
        if not self.head:
            return None
        other_head = self.algorithm.new_node(data=self.algorithm.get_data(self.head))
        other_node = other_head
        self_node = self.algorithm.get_next(self.head)
        while self_node:
            self.algorithm.set_next(
                node=other_node,
                next_node=self.algorithm.new_node(self.algorithm.get_data(self_node))
            )
            self_node = self.algorithm.get_next(self_node)
            other_node = self.algorithm.get_next(other_node)
        return SinglyLinkedList(other_head, self.algorithm.data_name, self.algorithm.next_name)

    def is_copied(self, other_list: SinglyLinkedList) -> bool:
        other_head = other_list.head
        if self.head:
            self_node, other_node = self.head, other_head
            while self_node:
                if self.algorithm.get_data(self_node) != self.algorithm.get_data(other_node):
                    return False
                self_node = self.algorithm.get_next(self_node)
                other_node = self.algorithm.get_next(other_node)
            return True
        else:
            return other_head == None

    def reverse(self):
        if self.head:
            previous_node, current_node, next_node = None, self.head, self.algorithm.get_next(self.head)
            while next_node:
                self.algorithm.set_next(node=current_node, next_node=previous_node)
                previous_node = current_node
                current_node = next_node
                next_node = self.algorithm.get_next(next_node)
            self.head = current_node

    def delete(self, data_set: set):
        """ O(N) """
        previous_node, current_node = self.algorithm.new_node(next_node=self.head), self.head
        fake_head, count = previous_node, 0
        while current_node:
            if self.node_data(current_node) in data_set:
                self.algorithm.set_next(node=previous_node, next_node=self.algorithm.get_next(current_node))
                count += 1
            else:
                previous_node = current_node
            current_node = self.algorithm.get_next(current_node)
        self.head = self.algorithm.get_next(fake_head)
        self.size -= count

    def delete_nth_from_end(self, n: int = 1):
        """ O(N) """
        if n >= 1 and n <= self.size:
            fake_head = self.algorithm.new_node(next_node=head)
            fast_node, slow_node = self.head, fake_head
            for _ in range(n):  # move fast node n steps
                if not fast_node:
                    return
                fast_node = self.algorithm.get_next(fast_node)
            while fast_node:
                fast_node = self.algorithm.get_next(fast_node)
                slow_node = self.algorithm.get_next(slow_node)
            self.algorithm.set_next(node=slow_node, next_node=self.algorithm.get_next(slow_node, 2))
            self.algorithm.set_next(node=self.head, next_node=self.algorithm.get_next(fake_head))
            self.size -= 1
        else:
            raise IndexError(f"n = {n} is out of range [1, {self.size}]")

    def swap_pairs_of_nodes(self):
        if self.head and self.algorithm.get_next(self.head):
            # We need 3 nodes to swap a pair of nodes (second, third)
            # first second third
            #   |      |     |
            #  fake    n1 -> n2 -> n3 -> n4 -> ...
            fake_head = self.algorithm.new_node()
            first, second, third = fake_head, self.head, self.algorithm.get_next(self.head)
            while third:
                self.algorithm.set_next(node=first, next_node=third)
                self.algorithm.set_next(node=second, next_node=self.algorithm.get_next(third))
                self.algorithm.set_next(node=third, next_node=second)
                # first second third
                #   |      |     |
                #  fake    n1 <- n2    n3 -> n4 -> ...  
                #   |------|-----^     ^ 
                #          |-----------|
                third = self.algorithm.get_next(second)
                # first       second third
                #   |           |     |
                # fake -> n2 -> n1 -> n3 -> n4 -> ...
                if not third or not self.algorithm.get_next(third):
                    break
                first, second, third = second, third, self.algorithm.get_next(third)
                #             first second third
                #               |     |     |
                # fake -> n2 -> n1 -> n3 -> n4 -> ...
            self.head = self.algorithm.get_next(fake_head)

    def get_intersection_head(self, other_list: SinglyLinkedList):
        size_delta = self.size - other_list.size
        long_list_node, short_list_node = (self.head, other_list.head) if size_delta > 0 else (other_list.head, self.head)
        long_list_node = self.algorithm.get_next(node=long_list_node, steps=abs(size_delta))
        while short_list_node and short_list_node != long_list_node:
            short_list_node = self.algorithm.get_next(short_list_node)
            long_list_node = self.algorithm.get_next(long_list_node)
        return short_list_node

    def has_cycle(self):
        fast_node, slow_node = self.head, self.head
        while fast_node and self.algorithm.get_next(fast_node):
            fast_node = self.algorithm.get_next(self.algorithm.get_next(fast_node))
            slow_node = self.algorithm.get_next(slow_node)
            if fast_node == slow_node:
                return True
        return False

    def get_cycle_entrance(self):
        # https://en.wikipedia.org/wiki/Cycle_detection
        #   ┌─────── A ───────┐┌──── B ────┐
        #  n1 -> n2 -> ... -> n3 -> ... -> n4 <-- meeting point
        #                   ┌ ^            | ─┐   circle perimeter = C
        #                   │ |            v  │   B < C
        #                   │ n6 <- ... <- n5 │
        #                   └───── C - B ─────┘
        # When the fast meets the slow, the fast has walked N rounds in the circle, where N > 1 
        # (A + B) * 2 = A + B + C * N, since the fast is 2 times faster than the slow 
        # => A + B = C * N
        # => A = (C - B) + C * (N - 1)
        # C - B is the length from the meeting point (n4) to the entering point (n3)
        # so if one pointer start from head and another start from the meeting point at the same speed
        # they will meet at the entering point and the one in the circle will have walked (N - 1) rounds
        fast_node, slow_node = self.head, self.head
        while fast_node and self.algorithm.get_next(fast_node):
            fast_node = self.algorithm.get_next(self.algorithm.get_next(fast_node)) # 2 times faster
            slow_node = self.algorithm.get_next(slow_node)
            if fast_node == slow_node:
                slow_node = self.head
                while slow_node != fast_node:
                    fast_node = self.algorithm.get_next(fast_node) # same speed as slow
                    slow_node = self.algorithm.get_next(slow_node)
                return slow_node
        return None


class DoublyLinkedList:
    def __init__(self, head=None, tail=None, data_name: str = DATA_NAME, next_name: str = NEXT_NAME, prev_name: str = PREV_NAME):
        self.head = head
        self.tail = tail
        self.algorithm = DoublyLinkedListAlgorithm(data_name, next_name, prev_name)
        self.size = self.calculate_size(set_tail=(tail is None))

    def __len__(self):
        return self.size

    def __str__(self):
        return self.to_string()

    def calculate_size(self, set_tail=False) -> int:
        """ O(N) """
        size, node = 0, self.head
        while node:
            if set_tail and not self.algorithm.has_next(node):
                self.tail = node
            size, node = size + 1, self.algorithm.get_next(node)
        return size

    def to_string(self, include_end=True,
        forward_link: str = FORWARD_LINK, backward_link: str = BACKWARD_LINK, bidirection_link: str = BIDIRECTION_LINK
    ):
        printer = DoublyLinkedListPrinter(self.algorithm, forward_link, backward_link, bidirection_link)
        return printer.to_string(self.head, include_end)

    def print(self, include_end=True,
        forward_link: str = FORWARD_LINK, backward_link: str = BACKWARD_LINK, bidirection_link: str = BIDIRECTION_LINK
    ):
        print(self.to_string(include_end, forward_link, backward_link, bidirection_link))

    def add_to_head(self, data):
        """ O(1) """
        new_node = self.algorithm.new_node(data=data)
        self.add_node_to_head(new_node)

    def add_node_to_head(self, node):
        """ O(1) """
        self.algorithm.set_next(node=node, next_node=self.head)
        self.algorithm.set_prev(node=node, prev_node=None)
        if self.head:
            self.algorithm.set_prev(node=self.head, prev_node=node)
        else:
            self.tail = node
        self.head = node
        self.size += 1

    def add_to_tail(self, data):
        """ O(1) """
        new_node = self.algorithm.new_node(data=data, prev_node=self.tail)
        self.add_node_to_tail(new_node)

    def add_node_to_tail(self, node):
        """ O(1) """
        self.algorithm.set_next(node=node, next_node=None)
        self.algorithm.set_prev(node=node, prev_node=self.tail)
        if self.tail:
            self.algorithm.set_next(node=self.tail, next_node=node)
        else:
            self.head = node
        self.tail = node
        self.size += 1

    def peek_head(self):
        """ O(1) """
        if not self.head:
            raise IndexError("Peek head at an empty list")
        return self.algorithm.get_data(self.head)

    def peek_tail(self):
        """ O(1) """
        if not self.tail:
            raise IndexError("Peek tail at an empty list")
        return self.algorithm.get_data(self.tail)

    def pop_head(self):
        """ O(1) """
        if not self.head:
            raise IndexError("Pop head from an empty list")
        data = self.algorithm.get_data(self.head)
        self.head = self.algorithm.get_next(self.head)
        if self.head:
            self.algorithm.set_prev(node=self.head, prev_node=None)
        else:
            self.tail = None
        self.size -= 1
        return data

    def pop_tail(self):
        """ O(1) """
        if not self.tail:
            raise IndexError("Pop tail from an empty list")
        data = self.algorithm.get_data(self.tail)
        self.tail = self.algorithm.get_prev(self.tail)
        if self.tail:
            self.algorithm.set_next(node=self.tail, next_node=None)
        else:
            self.head = None
        self.size -= 1
        return data

    def detach_node(self, node):
        if node == self.head:
            self.pop_head()
        elif node == self.tail:
            self.pop_tail()
        else:
            prev_node = self.algorithm.get_prev(node)
            next_node = self.algorithm.get_next(node)
            self.algorithm.set_next(node=prev_node, next_node=next_node)
            self.algorithm.set_prev(node=next_node, prev_node=prev_node)
            self.size -= 1




from typing import List


def init_matrix(row: int, col: int, init=None) -> list[list]:
    return [[init] * col for _ in range(row)]


class MatrixIterator:
    def __init__(self, matrix: List[list], row: int = 0, col: int = 0, direction: str="horizontal"):
        self.matrix = matrix
        self.row_len = len(matrix)
        self.col_len = len(matrix[0])
        self.row = row
        self.col = col
        self.direction = direction
        self.valid_directions = {"horizontal", "vertical", "ascending-diagonal", "descending-diagonal"}
        if direction not in self.valid_directions:
            raise ValueError(f"Invalid direction \"{self.direction}\", choose from {self.valid_directions}")

    def __iter__(self):
        return self

    def __next__(self):
        if self.direction == "horizontal":
            if self.col < self.col_len:
                data = self.matrix[self.row][self.col]
                self.col += 1
                return data
        elif self.direction == "vertical":
            if self.row < self.row_len:
                data = self.matrix[self.row][self.col]
                self.row += 1
                return data
        elif self.direction == "ascending-diagonal":
            if self.row >= 0 and self.col < self.col_len:
                data = self.matrix[self.row][self.col]
                self.row -= 1
                self.col += 1
                return data
        else:  # "descending-diagonal"
            if self.row < self.row_len and self.col < self.col_len:
                data = self.matrix[self.row][self.col]
                self.row += 1
                self.col += 1
                return data
        raise StopIteration



class Trie:
    class Node:
        def __init__(self, data=None, count=1):
            self.data = data
            self.count = count
            self.is_end = False
            self.children = None  # {data: Node}

    def __init__(self):
        self.root = self.Node(data="^", count=0)

    def clear(self):
        if self.root.count > 0:
            self.root = self.Node(data="^", count=0)

    def get_child(self, node, data):
        if not node.children or data not in node.children:
            return None
        return node.children[data]

    def size(self):
        return self.root.count

    def __str__(self):

        def _to_string_pre_order(node, str_list, result):
            if node:
                if node.is_end:
                    str_list.append(f"{node.data}:{node.count}:$")
                else:
                    str_list.append(f"{node.data}:{node.count}")
                if not node.children:
                    result.append(" -> ".join(str_list))
                else:
                    for child_node in node.children.values():
                        _to_string_pre_order(child_node, str_list, result)
                str_list.pop()

        result = list()
        _to_string_pre_order(self.root, list(), result)
        return "\n".join(result) + "\n"

    def print(self):
        print(self, end="")

    def add(self, prefix: list):
        if prefix is None:
            raise ValueError(f"prefix cannot be None")
        node = self.root
        node.count += 1
        for data in prefix:
            next_node = self.get_child(node, data)
            if next_node:
                node = next_node
                node.count += 1
            else:
                new_node = self.Node(data=data, count=1)
                if not node.children:
                    node.children = dict()
                node.children[data] = new_node
                node = new_node
        node.is_end = True

    def contains(self, prefix: list, strict=False):
        if prefix is None:
            raise ValueError(f"prefix cannot be None")
        node = self.root
        for data in prefix:
            node = self.get_child(node, data)
            if not node:
                return False
        if strict:
            return node.is_end
        return True

    def longest_common_prefix(self):
        node, prefix = self.root, list()
        while node:
            next_node = None
            if node.children:
                for child_node in node.children.values():
                    if child_node and child_node.count == self.root.count:
                        prefix.append(child_node.data)
                        next_node = child_node
                        break
            node = next_node
        return prefix

    def prefix_wildcard(self, prefix: list = None):

        def _prefix_wildcard_pre_order(node, data_list, result):
            if node:
                if node != self.root:
                    data_list.append(node.data)
                if node.is_end:
                    result.append(data_list.copy())
                if node.children:
                    for child_node in node.children.values():
                        _prefix_wildcard_pre_order(child_node, data_list, result)
                if len(data_list) > 0:
                    data_list.pop()

        result = list()
        if not prefix:
            _prefix_wildcard_pre_order(self.root, list(), result)
        else:
            node = self.root
            for data in prefix:
                node = self.get_child(node, data)
                if not node:
                    return None
            _prefix_wildcard_pre_order(node, list(prefix)[:-1:].copy(), result)
        return result   


class SuffixTrie(Trie):
    def __init__(self, data_list: list = None):
        super().__init__()
        self.build(data_list)

    def build(self, data_list: list):
        self.clear()
        for i in range(len(data_list)):
            self.add(data_list[i:])




from ezcode.list.const import BACKWARD_LINK
from ezcode.list.linked_list import DoublyLinkedList


class Queue:
    def __init__(self):
        self.doubly_linked_list = DoublyLinkedList()

    def __len__(self):
        return len(self.doubly_linked_list)

    def __str__(self):
        return self.doubly_linked_list.to_string(
            include_end=False,
            backward_link=BACKWARD_LINK,
            bidirection_link=BACKWARD_LINK
        )

    def print(self):
        print(self)

    def push(self, data):
        self.doubly_linked_list.add_to_tail(data)

    def pop(self):
        if len(self) == 0:
            raise IndexError("Pop from an empty queue")
        return self.doubly_linked_list.pop_head()

    def peek(self):
        if len(self) == 0:
            raise IndexError("Peek at an empty queue")
        return self.doubly_linked_list.peek_head()


class MonotonicQueue(Queue):
    def __init__(self, is_increasing: bool = True):
        """
        Monotonic Increasing Queue: min head
        Monotonic Decreasing Queue: max head
        """
        super().__init__()
        self.is_increasing = is_increasing

    def push(self, data):
        if self.is_increasing:
            while len(self) > 0 and self.doubly_linked_list.peek_tail() > data:
                self.doubly_linked_list.pop_tail()
        else:
            while len(self) > 0 and self.doubly_linked_list.peek_tail() < data:
                self.doubly_linked_list.pop_tail()
        self.doubly_linked_list.add_to_tail(data)




from ezcode.list.const import DATA_NAME, NEXT_NAME, FORWARD_LINK, BACKWARD_LINK, BIDIRECTION_LINK
from ezcode.list.algorithm import SinglyLinkedListAlgorithm, DoublyLinkedListAlgorithm


class SinglyLinkedListPrinter:
    def __init__(self, algorithm: SinglyLinkedListAlgorithm,
        forward_link: str = FORWARD_LINK, backward_link: str = BACKWARD_LINK
    ):
        self.algorithm = algorithm
        self.forward_link = forward_link
        self.backward_link = backward_link

    def to_string(self, node, reverse=False, include_end=True):
        if not node:
            return "None" if include_end else ""
        string, link = "", self.backward_link if reverse else self.forward_link
        while self.algorithm.has_next(node):
            new_string = f"{self.algorithm.get_data(node)}"
            string = link + new_string + string if reverse else string + new_string + link
            node = self.algorithm.get_next(node)
        new_string = f"{self.algorithm.get_data(node)}"
        if include_end:
            return  "None" + link + new_string + string if reverse else string + new_string + link + "None"
        else:
            return new_string + string if reverse else string + new_string

    def print(self, node, reverse=False, include_end=True):
        print(self.to_string(node, reverse, include_end))


class DoublyLinkedListPrinter:
    def __init__(self, algorithm: DoublyLinkedListAlgorithm,
        forward_link: str = FORWARD_LINK, backward_link: str = BACKWARD_LINK, bidirection_link: str = BIDIRECTION_LINK):
        self.algorithm = algorithm
        self.forward_link = forward_link
        self.backward_link = backward_link
        self.bidirection_link = bidirection_link

    def to_string(self, node, include_end=True):
        if not node:
            return "None" if include_end else ""
        string, backward_node = "", self.algorithm.get_prev(node)
        while self.algorithm.has_next(node):
            string += f"{self.algorithm.get_data(node)}{self.bidirection_link}"
            node = self.algorithm.get_next(node)
        string += f"{self.algorithm.get_data(node)}{self.forward_link}None" if include_end else f"{self.algorithm.get_data(node)}"
        if not backward_node:
            return f"None{self.backward_link}{string}" if include_end else string
        node = backward_node
        while node:
            string = f"{self.algorithm.get_data(node)}{self.bidirection_link}" + string
            node = self.algorithm.get_prev(node)
        return f"None{self.backward_link}{string}" if include_end else string

    def print(self, node, include_end=True):
        print(self.to_string(node, include_end))






from ezcode.list.linked_list import SinglyLinkedList


class Stack:
    def __init__(self):
        self.singly_linked_list = SinglyLinkedList()

    def __len__(self):
        return len(self.singly_linked_list)

    def __str__(self):
        return self.singly_linked_list.to_string(reverse=True)

    def print(self):
        print(self)

    def push(self, data):
        self.singly_linked_list.add_to_head(data)

    def pop(self):
        if len(self) == 0:
            raise IndexError("Pop from an empty stack")
        return self.singly_linked_list.pop_head()

    def peek(self):
        if len(self) == 0:
            raise IndexError("Peek at an empty stack")
        return self.singly_linked_list.peek_head()


class MinStack:
    def __init__(self):
        self.stack = Stack()
        self.min_stack = Stack()

    def get_min(self):
        return self.min_stack.peek()

    def push(self, data):
        if len(self.min_stack) == 0 or data <= self.min_stack.peek():
            self.min_stack.push(data)
        self.stack.push(data)

    def peek(self):
        return self.stack.peek()

    def pop(self):
        if self.stack.peek() == self.min_stack.peek():
            self.min_stack.pop()
        return self.stack.pop()


class MaxStack:
    def __init__(self):
        self.stack = Stack()
        self.max_stack = Stack()

    def get_max(self):
        return self.max_stack.peek()

    def push(self, data):
        if len(self.max_stack) == 0 or data >= self.max_stack.peek():
            self.max_stack.push(data)
        self.stack.push(data)

    def peek(self):
        return self.stack.peek()

    def pop(self):
        if self.stack.peek() == self.max_stack.peek():
            self.max_stack.pop()
        return self.stack.pop()



from ezcode.array.utils import swap


def rotate(array, shifts, is_left_rotate):

    def _rotate(array, begin, end):
        while begin < end:
            swap(array, begin, end)
            begin += 1
            end -= 1

    if array:
        size = len(array)
        shifts = shifts % size if is_left_rotate else size - shifts % size
        if shifts > 0:
            _rotate(array, 0, shifts - 1)
            _rotate(array, shifts, size - 1)
            _rotate(array, 0, size - 1)


# def is_rotated_sorted_array(array, is_ascending=True) -> bool:
#     not_in_order = 0
#     for i in range(len(array) - 1):
#         if not_in_order > 0 and (is_ascending and array[i] > array[0]) or (not is_ascending and array[i] < array[0]):
#             return False
#         if (is_ascending and array[i] > array[i + 1]) or (not is_ascending and array[i] < array[i + 1]):
#             not_in_order += 1
#     if not_in_order == 1:
#         if is_ascending:
#             return array[len(array) - 1] <= array[0]
#         else:
#             return array[len(array) - 1] >= array[0]
#     else:
#         return not_in_order == 0
# 
# 
# def find_min_index_in_rotated_sorted_array(array, is_ascending=True):
#     begin, end = 0, len(array) - 1
#     while begin < end:
#         mid = begin + (end - begin) // 2;
#         if mid == begin:
#             return begin if array[begin] < array[end] else end
#         if is_ascending and array[mid] == array[end]:  # Handle the case with duplicates
#             end -= 1
#         elif not is_ascending and array[mid] == array[begin]:
#             begin += 1
#         elif (is_ascending and array[mid] < array[end]) or (not is_ascending and array[mid] > array[begin]):
#             end = mid
#         else:
#             begin = mid
#     return begin 
# 
# 
# def binary_search_rotated(array, target, is_ascending=True, is_left_most=True):
#     min_index = find_min_index_in_rotated_sorted_array(array, is_ascending)
#     if (is_ascending and min_index == 0) or (not is_ascending and min_index == len(array) - 1):
#         return binary_search(array, target, is_ascending, is_left_most)
#     if is_ascending:
#         if target >= array[0]:
#             return _binary_search(array, 0, min_index - 1, target, is_ascending, is_left_most)
#         else:
#             return _binary_search(array, min_index, len(array) - 1, target, is_ascending, is_left_most)
#     else:
#         if target <= array[0]: 
#             return _binary_search(array, 0, min_index, target, is_ascending, is_left_most)
#         else:
#             return _binary_search(array, min_index + 1, len(array) - 1, target, is_ascending, is_left_most)





import setuptools

with open("README.md", "r", encoding="utf-8") as file_handler:
    long_description = file_handler.read()

setuptools.setup(
    name="ezcode",                                  # This is the name of the package
    version="0.0.31",                               # The initial release version
    author="Zheng Gao",                             # Full name of the author
    description="Easy Algorithm & Data Structure",
    url="https://github.com/zheng-gao/ez_code",
    project_urls={
        "Bug Tracker": "https://github.com/zheng-gao/ez_code/issues",
    },
    long_description=long_description,              # Long description read from the the readme file
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],                                              # Information to filter the project on PyPi website
    python_requires=">=3.7",                        # Minimum version requirement of the package
    py_modules=["ezcode"],                          # Name of the python package
    package_dir={"": "src"},                        # Directory of the source code of the package
    packages=setuptools.find_packages(where="src"), # List of all python modules to be installed
    install_requires=[]                             # Install other dependencies if any
)



from ezcode.array.utils import swap


def permutation_size(total_size: int, selection_size: int) -> int:
    """ Not for duplicate items: P(N,r) = N!/(N-r)! """
    if selection_size > total_size:
        raise ValueError(f"selection_size:{selection_size} cannot be greater than total_size:{total_size}")
    if selection_size < 0:
        raise ValueError(f"selection_size:{selection_size} cannot be negative")
    result = 1
    for i in range(total_size - selection_size + 1, total_size + 1):
        result *= i
    return result


def permutations(selection_size: int, items: list) -> list:
    def _permutations(selection_size: int, items: list, selected_indices: set(), permutation: list, result: list):
        if len(permutation) == selection_size:
            result.append(permutation.copy())
            return
        selected_items = set()
        for i in range(len(items)):
            if i not in selected_indices and items[i] not in selected_items:  # check for duplicate indices and duplicate items
                selected_indices.add(i)
                selected_items.add(items[i])
                permutation.append(items[i])
                _permutations(selection_size, items, selected_indices, permutation, result)
                permutation.pop()
                selected_indices.remove(i)

    if items is None:
        return None
    if selection_size > len(items):
        raise ValueError(f"selection_size:{selection_size} cannot be greater than len(items):{len(items)}")
    if selection_size < 0:
        raise ValueError(f"selection_size:{selection_size} cannot be negative")
    result = list()
    _permutations(selection_size, items, set(), list(), result)
    return result


def all_items_permutations(items: list) -> list:
    """ Equals to permutations(len(items, items)) """
    def _permutations_with_all_items(items: list, current_index, result: list):
        if current_index == len(items) - 1:
            result.append(items.copy())
            return
        selected_items = set()
        for next_index in range(current_index, len(items)):
            if items[next_index] not in selected_items:  # check for duplicate items
                selected_items.add(items[next_index])
                swap(items, current_index, next_index)
                _permutations_with_all_items(items, current_index + 1, result)
                swap(items, current_index, next_index)

    if items is None:
        return None
    if len(items) == 0:
        return [[]]
    result = list()
    _permutations_with_all_items(items, 0, result)
    return result


def combination_size(total_size: int, selection_size: int) -> int:
    """ C(N,r) = C(N,N-r) = A(N,r)/r! = N!/[r! * (N-r)!] """
    if selection_size > total_size:
        raise ValueError(f"selection_size:{selection_size} cannot be greater than total_size:{total_size}")
    if selection_size < 0:
        raise ValueError(f"selection_size:{selection_size} cannot be negative")
    result = permutation_size(total_size, selection_size)
    for i in range(1, selection_size + 1):
        result //= i
    return result


def combinations(selection_size: int, items: list):
    def _combinations(selection_size: int, items: list, start_index: int, combination: list, result: list):
        if len(combination) == selection_size:
            result.append(combination.copy())
            return
        selected_items = set()
        end_index = len(items) - (selection_size - len(combination))  # not enough indices left, terminate recursion early
        for i in range(start_index, end_index + 1):
            if items[i] not in selected_items:  # check for duplicate items
                selected_items.add(items[i])
                combination.append(items[i])
                _combinations(selection_size, items, i + 1, combination, result)
                combination.pop()

    if items is None:
        return None
    if selection_size > len(items):
        raise ValueError(f"selection_size:{selection_size} cannot be greater than len(items):{len(items)}")
    if selection_size < 0:
        raise ValueError(f"selection_size:{selection_size} cannot be negative")
    result = list()
    items.sort()  # for duplicate items
    _combinations(selection_size, items, 0, list(), result)
    return result


def all_subsets(items: list, has_duplicate=True):
    """ Equals to combinations(i, items) for i in range(len(items) + 1) """
    def _all_subsets_recursion(items: list, start_index: int, subset: list, result: list):
        if start_index > len(items):
            return
        result.append(subset.copy())
        selected_items = set()
        for i in range(start_index, len(items)):
            if items[i] not in selected_items:  # check for duplicate items
                selected_items.add(items[i])
                subset.append(items[i])
                _all_subsets_recursion(items, i + 1, subset, result)
                subset.pop()

    def _all_subsets_iteration(items: list, result: list):
        """ 2^N subsets, Not for duplicate items """
        number_range = 1 << len(items)
        for i in range(number_range):
            subset = list()
            for shift in range(len(items)):
                if ((i >> shift) & 1) == 1:
                    subset.append(items[shift])
            result.append(subset.copy())

    if items is None:
        return None
    result = list()
    if has_duplicate:
        items.sort()  # for duplicate items
        _all_subsets_recursion(items, 0, list(), result)
    else:
        _all_subsets_iteration(items, result)
    result.sort(key=lambda x: len(x)) # nice to have
    return result










import heapq


class PriorityQueue:
    def __init__(self, data: list = None, min_heap: bool = True):
        self.min_heap = min_heap
        self.heap = data.copy() if data else list()  # (priority, key)
        if self.min_heap:
            heapq.heapify(self.heap)
        else:
            heapq._heapify_max(self.heap)

    def __len__(self):
        return len(self.heap)

    def __str__(self):
        return str(self.heap)

    def peek(self):
        """ O(1) """
        if len(self) <= 0:
            raise IndexError("Peek at an empty queue")
        return self.heap[0]

    def push(self, data):
        """ O(logN) """
        if self.min_heap:
            heapq.heappush(self.heap, data)
        else:
            self.heap.append(data)
            heapq._siftdown_max(self.heap, 0, len(self.heap) - 1)  # python siftdown is from leaf to root

    def pop(self):
        """ O(logN) """
        if len(self) <= 0:
            raise IndexError("Pop on an empty queue")
        return heapq.heappop(self.heap) if self.min_heap else heapq._heappop_max(self.heap)


class PriorityMap:
    def __init__(self, map: dict = None, min_heap: bool = True):
        self.min_heap = min_heap
        self.heap = list() # (priority, key)
        self.map = dict() # <key, heap_index>
        if map:
            for key, priority in map.items():
                self.push(priority, key)

    def __len__(self):
        return len(self.heap)

    def __str__(self):
        return str(self.heap)

    def __contains__(self, key):
        return key in self.map

    def peek(self):
        """ O(1) """
        if len(self) <= 0:
            raise IndexError("Peek at an empty queue")
        return self.heap[0]

    def get_priority(self, key):
        """ O(1) """
        if key not in self:
            raise KeyError(f"{key} not found")
        return self.heap[self.map[key]][0]

    def push(self, *priority_key):
        """ O(logN) """
        priority, key = priority_key[0] if type(priority_key[0]) is tuple else (priority_key[0], priority_key[1])
        if key in self:
            self.update(priority, key)
        else:
            self.heap.append((priority, key))
            index = len(self.heap) - 1
            self.map[key] = index
            self._sift_down(index)

    def pop(self):
        """ O(logN) """
        if len(self) <= 0:
            raise IndexError("Pop from an empty queue")
        elif len(self) == 1:
            del self.map[self.heap[0][1]]
            return self.heap.pop()
        else:
            top_item = self.heap[0]
            tail_item = self.heap[-1]
            self.heap[0] = tail_item
            self.map[tail_item[1]] = 0
            self.heap.pop()
            del self.map[top_item[1]]
            self._sift_up(0)
            return top_item

    def delete(self, key):
        """ O(logN) """
        if key not in self:
            raise KeyError(f"{key} not found")
        index = self.map[key]
        tail_item = self.heap[-1]
        self.map[tail_item[1]] = index
        self.heap[index] = tail_item
        del self.map[key]
        self.heap.pop()
        self._sift_up(index)

    def update(self, *priority_key):
        """ O(logN) """
        priority, key = priority_key[0] if type(priority_key[0]) is tuple else (priority_key[0], priority_key[1])
        if key not in self:
            raise KeyError(f"{key} not found")
        index = self.map[key]
        old_item = self.heap[index]
        if old_item[0] != priority:
            self.heap[index] = (priority, key)
            if (self.min_heap and priority < old_item[0]) or (not self.min_heap and priority > old_item[0]):
                self._sift_down(index)  # python sift down is from leaf to root
            else:
                self._sift_up(index)  # python sift up is from root to leaf

    def _sift_down(self, index: int):
        """ python sift down is from leaf to root, O(logN) """
        new_item = self.heap[index]
        while index > 0:
            parent_index = (index - 1) >> 1
            parent = self.heap[parent_index]
            if (self.min_heap and new_item[0] < parent[0]) or (not self.min_heap and new_item[0] > parent[0]):
                self.heap[index] = parent
                self.map[parent[1]] = index
                index = parent_index
            else:
                break
        self.heap[index] = new_item
        self.map[new_item[1]] = index

    def _sift_up(self, index: int):
        """ python sift up is from root to leaf, O(logN) """
        end_index = len(self.heap) - 1
        new_item = self.heap[index]
        left_index = (index << 1) + 1
        while left_index <= end_index:
            right_index = left_index + 1
            child_index = left_index
            if right_index <= end_index:
                left_child, right_child = self.heap[left_index], self.heap[right_index]
                if (self.min_heap and right_child[0] < left_child[0]) or (not self.min_heap and right_child[0] > left_child[0]):
                    child_index = right_index
            child = self.heap[child_index]
            if (self.min_heap and child[0] < new_item[0]) or (not self.min_heap and child[0] > new_item[0]):
                self.heap[index] = child
                self.map[child[1]] = index
                index = child_index
                left_index = (index << 1) + 1
            else:
                break
        self.heap[index] = new_item
        self.map[new_item[1]] = index




